"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const cli_ux_1 = require("cli-ux");
const chalk = require("chalk");
const inquirer = require("inquirer");
const _ = require("lodash");
const openapi_client_axios_1 = require("openapi-client-axios");
const definition_1 = require("../common/definition");
const commonFlags = require("../common/flags");
const debug_1 = require("debug");
const utils_1 = require("../common/utils");
const debug = debug_1.default('cmd');
class Call extends command_1.Command {
    async run() {
        const { args, flags } = this.parse(Call);
        const { dereference, validate, bundle, header } = flags;
        const definition = definition_1.resolveDefinition(args.definition);
        if (!definition) {
            this.error('Please load a definition file', { exit: 1 });
        }
        let document;
        try {
            document = await definition_1.parseDefinition({
                definition,
                dereference,
                bundle,
                validate,
                servers: flags.server,
                header,
                induceServers: true,
            });
        }
        catch (err) {
            this.error(err, { exit: 1 });
        }
        const api = new openapi_client_axios_1.default({ definition: document });
        const client = await api.init();
        // select operation
        let operationId = flags.operation;
        if (!operationId) {
            const res = await inquirer.prompt([
                {
                    name: 'operation',
                    message: 'select operation',
                    type: 'list',
                    choices: api.getOperations().map((op) => {
                        const { operationId: id, summary, description, method, path } = op;
                        let name = `${method.toUpperCase()} ${path}`;
                        if (summary) {
                            name = `${name} - ${summary}`;
                        }
                        else if (description) {
                            name = `${name} - ${description}`;
                        }
                        if (id) {
                            name = `${name} (${id})`;
                        }
                        return { name, value: id };
                    }),
                },
            ]);
            operationId = res.operation;
        }
        const operation = api.getOperation(operationId);
        if (!operation) {
            this.error(`operationId ${operationId} not found`);
        }
        // fill params
        const params = {};
        for (const param of flags.param || []) {
            const [key, value] = param.split('=');
            params[key.trim()] = value;
        }
        for (const p of operation.parameters || []) {
            const param = p;
            const { name, required, example } = param;
            if (!params[name]) {
                const value = await cli_ux_1.default.prompt(name, { required, default: example });
                params[name] = value;
            }
        }
        // add request headers
        const config = { headers: utils_1.parseHeaderFlag(header) };
        // handle request body
        const data = flags.data;
        let res;
        try {
            const request = api.getRequestConfigForOperation(operation, [params, data, config]);
            debug(request);
            if (operationId)
                console.warn(chalk.bold(operationId));
            console.warn(`${chalk.green(request.method.toUpperCase())} ${request.url}`);
            res = await client[operationId](params, data, config);
        }
        catch (err) {
            if (err.response) {
                res = err.response;
            }
            else {
                this.error(err.message, { exit: false });
            }
        }
        // output response fields
        if (flags.include) {
            this.log(chalk.gray('RESPONSE META:'));
            this.log(JSON.stringify({
                code: res.status,
                status: res.statusText,
                headers: res.headers,
            }, null, 2));
            this.log(chalk.gray('RESPONSE BODY:'));
        }
        // output response body
        if (!_.isNil(res.data)) {
            try {
                this.log(JSON.stringify(res.data, null, 2));
            }
            catch (e) {
                this.log(res.data);
            }
        }
        else {
            console.warn(chalk.gray('(empty response)'));
        }
    }
}
exports.default = Call;
Call.description = 'Call API endpoints';
Call.examples = [
    `$ openapi call -o getPets`,
    `$ openapi call -o getPet -p id=1`,
    `$ openapi call -o createPet -d '{ "name": "Garfield" }'`,
];
Call.flags = Object.assign(Object.assign(Object.assign(Object.assign({}, commonFlags.help()), commonFlags.parseOpts()), commonFlags.apiRoot()), { operation: command_1.flags.string({ char: 'o', description: 'operationId', helpValue: 'operationId' }), param: command_1.flags.string({ char: 'p', description: 'parameter', helpValue: 'key=value', multiple: true }), data: command_1.flags.string({ char: 'd', description: 'request body' }), include: command_1.flags.boolean({
        char: 'i',
        description: 'include status code and response headers the output',
        default: false,
    }) });
Call.args = [
    {
        name: 'definition',
        description: 'input definition file',
    },
];
