"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.printSchemas = exports.printOperations = exports.getOperations = exports.printInfo = exports.resolveDefinition = exports.resolveConfigFile = exports.stringifyDocument = exports.OutputFormat = exports.parseDefinition = exports.CONFIG_FILENAME = void 0;
const SwaggerParser = require("@apidevtools/swagger-parser");
const lodash_1 = require("lodash");
const YAML = require("js-yaml");
const fs = require("fs");
const path = require("path");
const os_1 = require("os");
const utils_1 = require("./utils");
exports.CONFIG_FILENAME = '.openapiconfig';
async function parseDefinition({ definition, dereference, validate, bundle, servers, header, root, induceServers, }) {
    let method = SwaggerParser.parse;
    if (bundle) {
        method = SwaggerParser.bundle;
    }
    if (dereference) {
        method = SwaggerParser.dereference;
    }
    if (validate) {
        method = SwaggerParser.validate;
    }
    const parserOpts = {};
    // add headers
    if (header) {
        lodash_1.set(parserOpts, ['resolve', 'http', 'headers'], utils_1.parseHeaderFlag(header));
    }
    const document = await method.bind(SwaggerParser)(definition, parserOpts);
    // add servers
    if (servers) {
        const serverObjects = servers.map((url) => ({ url }));
        document.servers = document.servers ? [...serverObjects, ...document.servers] : serverObjects;
    }
    // induce the remote server from the definition parameter if needed
    if ((induceServers && definition.startsWith('http')) || definition.startsWith('//')) {
        document.servers = document.servers || [];
        const inputURL = new URL(definition);
        const server = document.servers[0];
        if (!server) {
            document.servers[0] = { url: `${inputURL.protocol}//${inputURL.host}` };
        }
        else if (!server.url.startsWith('http') && !server.url.startsWith('//')) {
            document.servers[0] = { url: `${inputURL.protocol}//${inputURL.host}${server.url}` };
        }
    }
    // override the api root for servers
    if (root) {
        if (!root.startsWith('/')) {
            root = `$/{root}`;
        }
        if (document.servers) {
            document.servers = document.servers.map((server) => {
                try {
                    const serverURL = new URL(server.url);
                    return Object.assign(Object.assign({}, server), { url: `${serverURL.protocol}//${serverURL.host}${root}` });
                }
                catch (_a) {
                    return Object.assign(Object.assign({}, server), { url: root });
                }
            });
        }
        else {
            document.servers = { url: root };
        }
    }
    return document;
}
exports.parseDefinition = parseDefinition;
var OutputFormat;
(function (OutputFormat) {
    OutputFormat["JSON"] = "json";
    OutputFormat["YAML"] = "yaml";
})(OutputFormat = exports.OutputFormat || (exports.OutputFormat = {}));
function stringifyDocument({ document, format }) {
    if (format === OutputFormat.JSON) {
        // JSON output
        return JSON.stringify(document, null, 2);
    }
    else {
        // YAML output
        return YAML.safeDump(document, { noRefs: true, lineWidth: 240, noArrayIndent: true });
    }
}
exports.stringifyDocument = stringifyDocument;
// walk backwards from cwd until homedir and check if CONFIG_FILENAME exists
function resolveConfigFile() {
    let dir = path.resolve(process.cwd());
    while (dir.length >= os_1.homedir().length) {
        const check = path.join(dir, exports.CONFIG_FILENAME);
        if (fs.existsSync(check)) {
            return path.join(dir, exports.CONFIG_FILENAME);
        }
        else {
            // walk backwards
            dir = path.resolve(path.join(dir, '..'));
        }
    }
}
exports.resolveConfigFile = resolveConfigFile;
function resolveDefinition(definitionArg) {
    // check definitionArg
    if (definitionArg && definitionArg !== 'CURRENT') {
        return definitionArg;
    }
    if (process.env.OPENAPI_DEFINITION && definitionArg !== 'CURRENT') {
        return process.env.OPENAPI_DEFINITION;
    }
    const configFile = resolveConfigFile();
    if (configFile) {
        const config = YAML.safeLoad(fs.readFileSync(configFile));
        return config.definition;
    }
}
exports.resolveDefinition = resolveDefinition;
function printInfo(document, ctx) {
    const { info, externalDocs } = document;
    if (info) {
        const { title, version, description, contact } = info;
        ctx.log(`title: ${title}`);
        ctx.log(`version: ${version}`);
        if (description) {
            ctx.log(`description: ${description}`);
        }
        if (contact) {
            if (contact.email && contact.name) {
                ctx.log(`contact: ${contact.name} <${contact.email}>`);
            }
            else if (contact.name) {
                ctx.log(`contact: ${contact.name}`);
            }
            else if (contact.email) {
                ctx.log(`contact: ${contact.email}`);
            }
            if (contact.url) {
                ctx.log(`website: ${contact.url}`);
            }
        }
    }
    if (externalDocs) {
        ctx.log(`docs: ${externalDocs.url}`);
    }
}
exports.printInfo = printInfo;
function getOperations(document) {
    const operations = [];
    for (const path in document.paths) {
        if (document.paths[path]) {
            for (const method in document.paths[path]) {
                if (document.paths[path][method]) {
                    operations.push(document.paths[path][method]);
                }
            }
        }
    }
    return lodash_1.uniqBy(operations, 'operationId');
}
exports.getOperations = getOperations;
function printOperations(document, ctx) {
    const operations = {};
    if (document.tags) {
        for (const tag of document.tags) {
            const { name, description } = tag;
            operations[name] = {
                description,
                routes: [],
            };
        }
    }
    for (const path in document.paths) {
        if (document.paths[path]) {
            for (const method in document.paths[path]) {
                if (document.paths[path][method]) {
                    const { operationId, summary, description, tags } = document.paths[path][method];
                    let route = `${method.toUpperCase()} ${path}`;
                    if (summary) {
                        route = `${route} - ${summary}`;
                    }
                    else if (description) {
                        route = `${route} - ${description}`;
                    }
                    if (operationId) {
                        route = `${route} (${operationId})`;
                    }
                    for (const tag of tags || ['default']) {
                        if (!operations[tag]) {
                            operations[tag] = { routes: [] };
                        }
                        operations[tag].routes.push(route);
                    }
                }
            }
        }
    }
    ctx.log('Operations:');
    for (const tag in operations) {
        if (operations[tag]) {
            const routes = operations[tag].routes;
            for (const route of routes) {
                ctx.log(`- ${route}`);
            }
        }
    }
}
exports.printOperations = printOperations;
function printSchemas(document, ctx) {
    const schemas = (document.components && document.components.schemas) || {};
    const count = Object.entries(schemas).length;
    if (count > 0) {
        ctx.log(`Schemas (${count}):`);
        for (const schema in schemas) {
            if (schemas[schema]) {
                ctx.log(`- ${schema}`);
            }
        }
    }
}
exports.printSchemas = printSchemas;
